@@ -26,9 +26,6 @@ import time
 
 from fixtures import Fixture, TempDir
 
-YIELD_TIME = 0.02  # How much time to sleep in busy loops.
-PROCESS_TIMEOUT = 5  # How long we wait for pgbouncer to startup and shutdown.
-
 def _allocate_ports(n=1):
     """Allocate `n` unused ports.
 
@@ -79,6 +76,7 @@ class PGBouncerFixture(Fixture):
         self.port = _allocate_ports()[0]
         self.configdir = self.useFixture(TempDir())
         self.auth_type = 'trust'
+        self.process_pid = None
         self.setUpConf()
         self.start()
 
@@ -88,6 +86,7 @@ class PGBouncerFixture(Fixture):
         self.authpath = os.path.join(self.configdir.path, 'users.txt')
         self.logpath = os.path.join(self.configdir.path, 'pgbouncer.log')
         self.pidpath = os.path.join(self.configdir.path, 'pgbouncer.pid')
+        self.outputpath = os.path.join(self.configdir.path, 'output')
         with open(self.inipath, 'wt') as inifile:
             inifile.write('[databases]\n')
             for item in self.databases.items():
@@ -112,19 +111,18 @@ class PGBouncerFixture(Fixture):
                 authfile.write('"%s" "%s"\n' % user_creds)
 
     def stop(self):
-        if self.process is None:
+        if self.process_pid is None:
             return
-        try:
-            self.process.terminate()
-            # Wait for the shutdown to occur
-            start = time.time()
-            stop = start + PROCESS_TIMEOUT
-            while self.process.poll() is None and time.time() < stop:
-                time.sleep(YIELD_TIME)
-        finally:
-            # If its not going away, we might want to raise an error,
-            # but for now it seems reliable.
-            self.process = None
+        os.kill(self.process_pid, signal.SIGTERM)
+        # Wait for the shutdown to occur
+        start = time.time()
+        stop = start + 5.0
+        while time.time() < stop:
+            if not os.path.isfile(self.pidpath):
+                self.process_pid = None
+                return
+        # If its not going away, we might want to raise an error, but for now
+        # it seems reliable.
 
     def start(self):
         self.addCleanup(self.stop)
@@ -139,22 +137,20 @@ class PGBouncerFixture(Fixture):
                 env = os.environ.copy()
                 env['PATH'] = ':'.join(path)
 
+        outputfile = open(self.outputpath, 'wt')
         self.process = subprocess.Popen(
-            [self.pgbouncer, '-q', self.inipath], env=env,
-            stderr=subprocess.STDOUT, stdin=subprocess.PIPE)
-        self.process.stdin.close()
-
-        # Wait up to 5 seconds for pgbouncer to launch start responding, or
-        # raise an exception.
-        now = time.time()
-        stop = now + PROCESS_TIMEOUT
-        while True:
-            try:
-                test_socket = socket.create_connection(
-                    ('127.0.0.1', self.port), stop - now)
+            [self.pgbouncer, '-d', self.inipath], env=env,
+            stdout=outputfile.fileno(), stderr=outputfile.fileno())
+        self.process.communicate()
+        # Wait up to 5 seconds for the pid file to exist
+        start = time.time()
+        stop = start + 5.0
+        while time.time() < stop:
+            if os.path.isfile(self.pidpath):
+                try:
+                    self.process_pid = int(file(self.pidpath, 'rt').read())
+                except ValueError:
+                    # Empty pid files -> ValueError.
+                    continue
                 return
-            except Exception:
-                now = time.time()
-                if now >= stop:
-                    raise
-            time.sleep(YIELD_TIME)
+        raise Exception('timeout waiting for pgbouncer to create pid file')
